@@ -22,14 +22,17 @@
 (test "q/abc" (pathname-directory "q/abc/.def.ghi"))
 (test "q/abc" (pathname-directory "q/abc/.ghi"))
 
-(test "./" (normalize-pathname "" 'unix))
-(test ".\\" (normalize-pathname "" 'windows))
+(test "." (normalize-pathname "" 'unix))
+(test "." (normalize-pathname "" 'windows))
 (test "\\..\\" (normalize-pathname "/../" 'windows))
-(test "\\." (normalize-pathname "/abc/../." 'windows))
-(test "/." (normalize-pathname "/" 'unix))
-(test "/." (normalize-pathname "/./" 'unix))
-(test "/." (normalize-pathname "/." 'unix))
-(test "./" (normalize-pathname "./" 'unix))
+(test "\\" (normalize-pathname "/abc/../." 'windows))
+(test "/" (normalize-pathname "/" 'unix))
+(test "/" (normalize-pathname "/." 'unix))
+(test "/" (normalize-pathname "/./" 'unix))
+(test "/" (normalize-pathname "/./." 'unix))
+(test "." (normalize-pathname "./" 'unix))
+(test "a" (normalize-pathname "./a"))
+(test "a" (normalize-pathname ".///a"))
 (test "a" (normalize-pathname "a"))
 (test "a/" (normalize-pathname "a/" 'unix))
 (test "a/b" (normalize-pathname "a/b" 'unix))
@@ -50,24 +53,25 @@
 (test "a/b" (normalize-pathname "a/b/c/d/../.." 'unix))
 (test "a/b/" (normalize-pathname "a/b/c/d/../../" 'unix))
 (test "../../foo" (normalize-pathname "../../foo" 'unix))
-(test "c:\\." (normalize-pathname "c:\\" 'windows))
+(test "c:\\" (normalize-pathname "c:\\" 'windows))
+(test "c:\\" (normalize-pathname "c:\\." 'windows))
+(test "c:\\" (normalize-pathname "c:\\.\\" 'windows))
+(test "c:\\" (normalize-pathname "c:\\.\\." 'windows))
 
-(define home (get-environment-variable "HOME"))
-
-(when home
-  (test (string-append home "/foo") (normalize-pathname "~/foo" 'unix))
-  (test "c:~/foo" (normalize-pathname "c:~/foo" 'unix))
-  (test (string-append home "\\foo") (normalize-pathname "c:~\\foo" 'windows)))
+(test "~/foo" (normalize-pathname "~/foo" 'unix))
+(test "c:~/foo" (normalize-pathname "c:~/foo" 'unix))
+(test "c:~\\foo" (normalize-pathname "c:~\\foo" 'windows))
 
 (assert (directory-null? "/.//"))
 (assert (directory-null? ""))
 (assert (not (directory-null? "//foo//")))
 
 (test '(#f "/" (".")) (receive (decompose-directory "/.//")))
-(test '(#f "\\" (".")) (receive (decompose-directory (normalize-pathname "/.//" 'windows))))
 (test '(#f "/" #f) (receive (decompose-directory "///\\///")))
 (test '(#f "/" ("foo")) (receive (decompose-directory "//foo//")))
 (test '(#f "/" ("foo" "bar")) (receive (decompose-directory "//foo//bar")))
+(test '(#f #f (".")) (receive (decompose-directory ".//")))
+(test '(#f #f ("." "foo")) (receive (decompose-directory ".//foo//")))
 (test '(#f #f (" " "foo" "bar")) (receive (decompose-directory " //foo//bar")))
 (test '(#f #f ("foo" "bar")) (receive (decompose-directory "foo//bar/")))
 
